package com.citygis.web.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.citygis.common.core.domain.AjaxResult;
import com.citygis.web.domain.*;
import com.citygis.web.mapper.TabAccessFeedbackMapper;
import com.citygis.web.service.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

import static com.citygis.common.utils.SecurityUtils.*;

/**
 * <p>
 * 接入反馈表 服务实现类
 * </p>
 *
 * @author citygis
 * @since 2025-01-07
 */
@Service
public class TabAccessFeedbackServiceImpl extends ServiceImpl<TabAccessFeedbackMapper, TabAccessFeedback> implements ITabAccessFeedbackService {

    @Resource
    ITabEtlService tabEtlService;

    @Resource
    ITabKafkaService tabKafkaService;

    @Resource
    ITabKbService tabKbService;

    @Resource
    ITabSftpService tabSftpService;

    @Resource
    ITabUrlService tabUrlService;

    @Resource
    ITabDatasourceMsgService tabDatasourceMsgService;


    // 提取一个公共方法来查询并设置 TabDatasourceMsg
    private <T> void setTabDatasourceMsgs(T tabType, Long accessTypeId) {
        if (tabType != null) {
            LambdaQueryWrapper<TabDatasourceMsg> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(TabDatasourceMsg::getAccessTypeId, accessTypeId);
            List<TabDatasourceMsg> datasourceMsgs = tabDatasourceMsgService.list(queryWrapper);
            // 根据实际类型，使用反射设置 TabDatasourceMsgs
            if (tabType instanceof TabEtl) {
                ((TabEtl) tabType).setTabDatasourceMsgs(datasourceMsgs);
            } else if (tabType instanceof TabKafka) {
                ((TabKafka) tabType).setTabDatasourceMsgs(datasourceMsgs);
            } else if (tabType instanceof TabKb) {
                ((TabKb) tabType).setTabDatasourceMsgs(datasourceMsgs);
            } else if (tabType instanceof TabSftp) {
                ((TabSftp) tabType).setTabDatasourceMsgs(datasourceMsgs);
            } else if (tabType instanceof TabUrl) {
                ((TabUrl) tabType).setTabDatasourceMsgs(datasourceMsgs);
            }
        }
    }

    @Override
    public AjaxResult getTabAccessFeedbackById(Long id) {
        TabAccessFeedback byId = this.getById(id);

        if (byId == null) {
            return AjaxResult.success("查询失败");
        }

        // 查询并设置 TabEtl
        TabEtl tabEtl = tabEtlService.getOne(new LambdaQueryWrapper<TabEtl>().eq(TabEtl::getAccessFeedbackId, id));
        setTabDatasourceMsgs(tabEtl, tabEtl != null ? tabEtl.getId() : null);
        byId.setEtl(tabEtl);

        // 查询并设置 TabKafka
        TabKafka tabKafka = tabKafkaService.getOne(new LambdaQueryWrapper<TabKafka>().eq(TabKafka::getAccessFeedbackId, id));
        setTabDatasourceMsgs(tabKafka, tabKafka != null ? tabKafka.getId() : null);
        byId.setKafka(tabKafka);

        // 查询并设置 TabKb
        TabKb tabKb = tabKbService.getOne(new LambdaQueryWrapper<TabKb>().eq(TabKb::getAccessFeedbackId, id));
        setTabDatasourceMsgs(tabKb, tabKb != null ? tabKb.getId() : null);
        byId.setKb(tabKb);

        // 查询并设置 TabSftp
        TabSftp tabSftp = tabSftpService.getOne(new LambdaQueryWrapper<TabSftp>().eq(TabSftp::getAccessFeedbackId, id));
        setTabDatasourceMsgs(tabSftp, tabSftp != null ? tabSftp.getId() : null);
        byId.setSftp(tabSftp);

        // 查询并设置 TabUrl
        TabUrl tabUrl = tabUrlService.getOne(new LambdaQueryWrapper<TabUrl>().eq(TabUrl::getAccessFeedbackId, id));
        setTabDatasourceMsgs(tabUrl, tabUrl != null ? tabUrl.getId() : null);
        byId.setUrl(tabUrl);

        return AjaxResult.success(byId);
    }


    @Override
    @Transactional
    public AjaxResult insertTabAccessFeedback(TabAccessFeedback tabAccessFeedback) {
        Snowflake snowflake = IdUtil.getSnowflake(1, 1);
        long id = snowflake.nextId();
        tabAccessFeedback.setId(id);
        tabAccessFeedback.setCreateBy(getUsername());
        tabAccessFeedback.setCreateTime(new Date());
        tabAccessFeedback.setUserId(getUserId());
        tabAccessFeedback.setDeptId(getDeptId());

        this.save(tabAccessFeedback);

        if (tabAccessFeedback.getEtl() != null) {
            TabEtl etl = tabAccessFeedback.getEtl();
            etl.setId(snowflake.nextId());
            etl.setAccessFeedbackId(id);
            tabEtlService.save(etl);

            List<TabDatasourceMsg> tabDatasourceMsgs = etl.getTabDatasourceMsgs();
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsg.setId(snowflake.nextId());
                    tabDatasourceMsg.setAccessTypeId(etl.getId());
                    tabDatasourceMsgService.save(tabDatasourceMsg);
                });
            }

        }

        if (tabAccessFeedback.getKafka() != null) {
            TabKafka kafka = tabAccessFeedback.getKafka();
            kafka.setId(snowflake.nextId());
            kafka.setAccessFeedbackId(id);
            tabKafkaService.save(kafka);

            List<TabDatasourceMsg> tabDatasourceMsgs = kafka.getTabDatasourceMsgs();
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsg.setId(snowflake.nextId());
                    tabDatasourceMsg.setAccessTypeId(kafka.getId());
                    tabDatasourceMsgService.save(tabDatasourceMsg);
                });
            }
        }

        if (tabAccessFeedback.getKb() != null) {
            TabKb kb = tabAccessFeedback.getKb();
            kb.setId(snowflake.nextId());
            kb.setAccessFeedbackId(id);
            tabKbService.save(kb);

            List<TabDatasourceMsg> tabDatasourceMsgs = kb.getTabDatasourceMsgs();
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsg.setId(snowflake.nextId());
                    tabDatasourceMsg.setAccessTypeId(kb.getId());
                    tabDatasourceMsgService.save(tabDatasourceMsg);
                });
            }
        }

        if (tabAccessFeedback.getSftp() != null) {
            TabSftp sftp = tabAccessFeedback.getSftp();
            sftp.setId(snowflake.nextId());
            sftp.setAccessFeedbackId(id);
            tabSftpService.save(sftp);

            List<TabDatasourceMsg> tabDatasourceMsgs = sftp.getTabDatasourceMsgs();
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsg.setId(snowflake.nextId());
                    tabDatasourceMsg.setAccessTypeId(sftp.getId());
                    tabDatasourceMsgService.save(tabDatasourceMsg);
                });
            }
        }

        if (tabAccessFeedback.getUrl() != null) {
            TabUrl url = tabAccessFeedback.getUrl();
            url.setId(snowflake.nextId());
            url.setAccessFeedbackId(id);
            tabUrlService.save(url);

            List<TabDatasourceMsg> tabDatasourceMsgs = url.getTabDatasourceMsgs();
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsg.setId(snowflake.nextId());
                    tabDatasourceMsg.setAccessTypeId(url.getId());
                    tabDatasourceMsgService.save(tabDatasourceMsg);
                });
            }
        }

        return AjaxResult.success("添加成功");
    }

    @Override
    public AjaxResult updateTabAccessFeedbackById(TabAccessFeedback tabAccessFeedback) {

        tabAccessFeedback.setUpdateBy(getUsername());
        tabAccessFeedback.setUpdateTime(new Date());
        this.updateById(tabAccessFeedback);

        if (tabAccessFeedback.getEtl() != null) {
            TabEtl etl = tabAccessFeedback.getEtl();
            tabEtlService.updateById(etl);

            List<TabDatasourceMsg> tabDatasourceMsgs = etl.getTabDatasourceMsgs();
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsgService.updateById(tabDatasourceMsg);
                });
            }
        }

        if (tabAccessFeedback.getKafka() != null) {
            TabKafka kafka = tabAccessFeedback.getKafka();
            tabKafkaService.updateById(kafka);

            List<TabDatasourceMsg> tabDatasourceMsgs = kafka.getTabDatasourceMsgs();
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsgService.updateById(tabDatasourceMsg);
                });
            }
        }

        if (tabAccessFeedback.getKb() != null) {
            TabKb kb = tabAccessFeedback.getKb();
            tabKbService.updateById(kb);

            List<TabDatasourceMsg> tabDatasourceMsgs = kb.getTabDatasourceMsgs();
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsgService.updateById(tabDatasourceMsg);
                });
            }
        }

        if (tabAccessFeedback.getSftp() != null) {
            TabSftp sftp = tabAccessFeedback.getSftp();
            tabSftpService.updateById(sftp);

            List<TabDatasourceMsg> tabDatasourceMsgs = sftp.getTabDatasourceMsgs();
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsgService.updateById(tabDatasourceMsg);
                });
            }
        }

        if (tabAccessFeedback.getUrl() != null) {
            TabUrl url = tabAccessFeedback.getUrl();
            tabUrlService.updateById(url);

            List<TabDatasourceMsg> tabDatasourceMsgs = url.getTabDatasourceMsgs();
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsgService.updateById(tabDatasourceMsg);
                });
            }
        }

        return AjaxResult.success("修改成功");
    }

    @Override
    public AjaxResult falseDeleteTabAccessFeedbackById(Long id) {
        TabAccessFeedback tabAccessFeedback = new TabAccessFeedback();
        tabAccessFeedback.setId(id);
        tabAccessFeedback.setIsDel(1);
        this.updateById(tabAccessFeedback);

        LambdaQueryWrapper<TabEtl> queryWrapperEtl = new LambdaQueryWrapper<>();
        queryWrapperEtl.eq(TabEtl::getAccessFeedbackId, id);
        TabEtl tabEtl = tabEtlService.getOne(queryWrapperEtl);
        if (tabEtl != null) {
            tabEtl.setIsDel(1);
            tabEtlService.updateById(tabEtl);

            LambdaQueryWrapper<TabDatasourceMsg> queryWrapperDatasourceMsg = new LambdaQueryWrapper<>();
            queryWrapperDatasourceMsg.eq(TabDatasourceMsg::getAccessTypeId, tabEtl.getId());
            List<TabDatasourceMsg> tabDatasourceMsgs = tabDatasourceMsgService.list(queryWrapperDatasourceMsg);
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsg.setIsDel(1);
                    tabDatasourceMsgService.updateById(tabDatasourceMsg);
                });
            }
        }


        LambdaQueryWrapper<TabKafka> queryWrapperKafka = new LambdaQueryWrapper<>();
        queryWrapperKafka.eq(TabKafka::getAccessFeedbackId, id);
        TabKafka tabKafka = tabKafkaService.getOne(queryWrapperKafka);
        if (tabKafka != null) {
            tabKafka.setIsDel(1);
            tabKafkaService.updateById(tabKafka);

            LambdaQueryWrapper<TabDatasourceMsg> queryWrapperDatasourceMsg = new LambdaQueryWrapper<>();
            queryWrapperDatasourceMsg.eq(TabDatasourceMsg::getAccessTypeId, tabKafka.getId());
            List<TabDatasourceMsg> tabDatasourceMsgs = tabDatasourceMsgService.list(queryWrapperDatasourceMsg);
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsg.setIsDel(1);
                    tabDatasourceMsgService.updateById(tabDatasourceMsg);
                });
            }
        }

        LambdaQueryWrapper<TabKb> queryWrapperKb = new LambdaQueryWrapper<>();
        queryWrapperKb.eq(TabKb::getAccessFeedbackId, id);
        TabKb tabKb = tabKbService.getOne(queryWrapperKb);
        if (tabKb != null) {
            tabKb.setIsDel(1);
            tabKbService.updateById(tabKb);

            LambdaQueryWrapper<TabDatasourceMsg> queryWrapperDatasourceMsg = new LambdaQueryWrapper<>();
            queryWrapperDatasourceMsg.eq(TabDatasourceMsg::getAccessTypeId, tabKb.getId());
            List<TabDatasourceMsg> tabDatasourceMsgs = tabDatasourceMsgService.list(queryWrapperDatasourceMsg);
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsg.setIsDel(1);
                    tabDatasourceMsgService.updateById(tabDatasourceMsg);
                });
            }
        }

        LambdaQueryWrapper<TabSftp> queryWrapperSftp = new LambdaQueryWrapper<>();
        queryWrapperSftp.eq(TabSftp::getAccessFeedbackId, id);
        TabSftp tabSftp = tabSftpService.getOne(queryWrapperSftp);
        if (tabSftp != null) {
            tabSftp.setIsDel(1);
            tabSftpService.updateById(tabSftp);

            LambdaQueryWrapper<TabDatasourceMsg> queryWrapperDatasourceMsg = new LambdaQueryWrapper<>();
            queryWrapperDatasourceMsg.eq(TabDatasourceMsg::getAccessTypeId, tabSftp.getId());
            List<TabDatasourceMsg> tabDatasourceMsgs = tabDatasourceMsgService.list(queryWrapperDatasourceMsg);
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsg.setIsDel(1);
                    tabDatasourceMsgService.updateById(tabDatasourceMsg);
                });
            }
        }

        LambdaQueryWrapper<TabUrl> queryWrapperUrl = new LambdaQueryWrapper<>();
        queryWrapperUrl.eq(TabUrl::getAccessFeedbackId, id);
        TabUrl tabUrl = tabUrlService.getOne(queryWrapperUrl);
        if (tabUrl != null) {
            tabUrl.setIsDel(1);
            tabUrlService.updateById(tabUrl);

            LambdaQueryWrapper<TabDatasourceMsg> queryWrapperDatasourceMsg = new LambdaQueryWrapper<>();
            queryWrapperDatasourceMsg.eq(TabDatasourceMsg::getAccessTypeId, tabUrl.getId());
            List<TabDatasourceMsg> tabDatasourceMsgs = tabDatasourceMsgService.list(queryWrapperDatasourceMsg);
            if (tabDatasourceMsgs != null) {
                tabDatasourceMsgs.forEach(tabDatasourceMsg -> {
                    tabDatasourceMsg.setIsDel(1);
                    tabDatasourceMsgService.updateById(tabDatasourceMsg);
                });
            }
        }

        return AjaxResult.success("删除成功");
    }
}